package smiley;

import java.awt.*;
import java.beans.*;
import java.util.*;

public class Smiley extends Component {

  public static final int HAPPY = 1;
  public static final int SAD = 2;
  public static final String HAPPY_STR = "Happy";
  public static final String SAD_STR = "Sad";
  private static final int BRIGHTNESS = 8;
  private static final int SHARPNESS = 8;
  private static final int MIN_SIZE = 32; // can't be changed
  private static final int MAX_SIZE = 256;
  private static final int DEFAULT_SIZE = 64;
  private int mood = HAPPY;
  private Color faceColor = Color.yellow;
  private Color featuresColor = Color.black;
  private Vector smileyListeners = new Vector();
  private PropertyChangeSupport changes =
    new PropertyChangeSupport(this);

  public Smiley() {
  }

  public synchronized void
    addSmileyListener(SmileyListener l) {
      smileyListeners.addElement(l);
  }

  public synchronized void
    removeSmileyListener(SmileyListener l) {
      smileyListeners.removeElement(l);
  }

  protected void notifyTargets() {
    Vector l;
    SmileyEvent s = new SmileyEvent(this);
    synchronized(this) {
      l = (Vector) smileyListeners.clone();
    }
    for (int i = 0; i < l.size(); i++) {
      SmileyListener sl = (SmileyListener) l.elementAt(i);
      sl.moodChanged(s);
    }
  }

  public void addPropertyChangeListener(
      PropertyChangeListener p) {
    changes.addPropertyChangeListener(p);
  }

  public void removePropertyChangeListener(
      PropertyChangeListener p) {
    changes.removePropertyChangeListener(p);
  }

  public synchronized void makeHappy() {
    mood = HAPPY;
    repaint();
    notifyTargets();
  }

  public synchronized void makeSad() {
    mood = SAD;
    repaint();
    notifyTargets();
  }

  public synchronized void setMood(int mood) {
    this.mood = mood;
    repaint();
    notifyTargets();
  }

  public int getMood() {
    return mood;
  }

  public synchronized void setDiameter(int d) {
    int old = getSize().width;
    d = getOptimalSize(d);
    setSize(d, d);
    repaint();
    changes.firePropertyChange(
      "diameter", new Integer(old),
      new Integer(d));
  }

  public int getDiameter() {
    return getCurrentSize();
  }

  private int getCurrentSize() {
    return Math.min(getSize().width, getSize().height);
  }

  private int getOptimalSize(int size) {
    if (size < MIN_SIZE)
      size = MIN_SIZE;
    else if (size > MAX_SIZE)
      size = MAX_SIZE;
    return size;
  }

  public void update(Graphics g) {
    paint(g);
  }

  public void paint(Graphics g) {
    g.setColor(faceColor);
    g.fillArc(0, 0,
      getCurrentSize() - 1,
      getCurrentSize() - 1, 0, 360);
    g.setColor(featuresColor);
    g.drawArc(0, 0,
      getCurrentSize() - 1,
      getCurrentSize() - 1, 0, 360);
    drawEyes(g);
    if (mood == HAPPY)
      drawSmile(g);
    else
      drawFrown(g);
  }

  private void drawEyes(Graphics g) {
    g.setColor(featuresColor);
    g.fillRoundRect(scaleIt(11), scaleIt(10),
      scaleEyes(), scaleEyes(),
      scaleCorner(), scaleCorner());
    g.fillRoundRect(scaleIt(18), scaleIt(10),
      scaleEyes(), scaleEyes(),
      scaleCorner(), scaleCorner());
  }

  private void drawSmile(Graphics g) {
    g.setColor(featuresColor);
    g.fillRoundRect(scaleIt(11), scaleIt(20),
      scaleIt(11), scaleIt(5),
      scaleCorner(), scaleCorner());
    g.setColor(faceColor);
    g.fillRoundRect(scaleIt(11), scaleIt(18),
      scaleIt(11), scaleIt(5),
      scaleCorner(), scaleCorner());
  }

  private void drawFrown(Graphics g) {
    g.setColor(featuresColor);
    g.fillRoundRect(scaleIt(11), scaleIt(20),
      scaleIt(11), scaleIt(5),
      scaleCorner(), scaleCorner());
    g.setColor(faceColor);
    g.fillRoundRect(scaleIt(11), scaleIt(22),
      scaleIt(11), scaleIt(5),
      scaleCorner(), scaleCorner());
  }

  private int scaleIt(int value) {
    return (value * getCurrentSize()) / MIN_SIZE;
  }

  private int scaleCorner() {
    return getCurrentSize() / SHARPNESS;
  }

  private int scaleEyes() {
    return getCurrentSize() / BRIGHTNESS;
  }

  public Dimension getMinimumSize() {
    return new Dimension(MIN_SIZE, MIN_SIZE);
  }

  public Dimension getMaximumSize() {
    return new Dimension(MAX_SIZE, MAX_SIZE);
  }

  public Dimension getPreferredSize() {
    int size = getCurrentSize();
    size = getOptimalSize(size);
    return new Dimension(size, size);
  }
}